/*
 * COPYRIGHT INFORMATION - DO NOT REMOVE
 * "Portions Copyright (c) 2000-2001 LinuxMagic Inc. All Rights Reserved.
 *
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the Free Source Code License Version
 * 1.0 (the 'License'). You may not use this file except in compliance with
 * the License. Please obtain a copy of the License at:
 *
 * http://www.linuxmagic.com/opensource/licensing/FSCL.txt
 *
 * and read it before using this file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND LINUXMAGIC HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT. Please see
 * the License for the specific language governing rights and limitations
 * under the License."
 *
 * Please read the terms of this license carefully. By using or downloading
 * this software or file, you are accepting and agreeing to the terms of this
 * license with LinuxMagic Inc. If you are agreeing to this license on behalf
 * of a company, you represent that you are authorized to bind the company to
 * such a license. If you do not meet this criterion or you do not agree to
 * any of the terms of this license, do NOT download, distribute, use or alter
 * this software or file in any way.
 *
 * Author: Burton Samograd (burton@wizard.ca)
 * CVS Id: $Id: lmsmtpd_config.c,v 1.30 2002/11/22 18:15:39 burton Exp $
 *
 * DO NOT MODIFY WITHOUT CONSULTING THE LICENSE
 */

/* C includes */
#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <sys/stat.h>

/* Library includes */
#include <liblm.h>

/* Project includes */
#include "lmsmtpd_config.h"

static spam_rule_config_t global_spam_rules;
static 	lm_config_string_t newline;
static 	lm_config_string_t emptystring;

/* Local function definitions */
static int lmsmtpd_config_read(int argc, char **argv);
static int lmsmtpd_config_setup_read_number(char *charkey, lm_config_number_t* number, lm_config_number_t defnumber);
static int lmsmtpd_config_setup_read_string(char *charkey, lm_config_string_t* string, lm_config_string_t defstring);
static int lmsmtpd_config_load_spam_config(spam_rule_config_t* globalrules, spam_rule_config_t *ruledata);

int lmsmtpd_config_load(int argc, char **argv)
{
	/* Initialize variables */
	config.smtpgreeting = NULL;
	config.localiphost = NULL;
	config.timeoutsmtpd = LM_SMTPD_DEFAULT_TIMEOUT;
	config.badmailfrom = NULL;
	config.rcpthosts = NULL;
	config.databytes = LM_SMTPD_DEFAULT_DATABYTES;
	config.tcplocalip = NULL;
	config.tcpremoteip= NULL;
	config.tcpremotehost= NULL;
	config.tcpremoteinfo= NULL;
	config.tcprelayclient= NULL;
	config.helohost = NULL;
	config.fakehelohost = 0;
	config.debuglevel = LM_LOG_LEVEL_ERROR;
	LM_STRING_NEW(config.qmailcontroldir, LM_SMTPD_QMAIL_CONTROL_DIR_DEFAULT);
	config.logfile = NULL;
	config.maxlinelen = LM_SMTPD_MAX_LINE_LENGTH_DEFAULT;
	config.maxhops = LM_SMTPD_MAX_HOPS_DEFAULT;
	config.maxrcpt = LM_SMTPD_MAX_RCPT_DEFAULT;
	config.rcptdelayat = LM_SMTPD_RCPT_DELAY_AT_DEFAULT;
	config.rcptdelayinc = LM_SMTPD_RCPT_DELAY_INC_DEFAULT;
	config.rcptdelaymax = LM_SMTPD_RCPT_DELAY_MAX_DEFAULT;
	LM_STRING_NEW(config.check_vdomain_prog, LM_SMTPD_CHECK_VDOMAIN_PROG_DEFAULT);
	config.check_valid_users = LM_SMTPD_CHECK_VALID_USERS_DEFAULT;
	config.per_user_spam_enable = LM_SMTPD_PER_USER_SPAM_ENABLE_DEFAULT;
	config.quarantine_on_hit = LM_SMTPD_QUARANTINE_ON_HIT_DEFAULT;
	config.ret_udne_on_hit = LM_SMTPD_RETURN_UDNE_ON_HIT_DEFAULT;
	LM_STRING_NEW(config.quarantine_ext,LM_SMTPD_QUARANTINE_EXT_DEFAULT);

	LM_STRING_NEW(newline, "\n");
	LM_STRING_NEW(emptystring, "");

	return lmsmtpd_config_read(argc, argv);
}

/* Clean up allocated memory */
int lmsmtpd_config_cleanup(void)
{
	LM_STRING_FREE(config.me);
	LM_STRING_FREE(config.smtpgreeting);
	LM_STRING_FREE(config.localiphost);
	LM_STRING_FREE(config.tcpremoteip);
	LM_STRING_FREE(config.tcplocalip);
	LM_STRING_FREE(config.rcpthosts);
	LM_STRING_FREE(config.badmailfrom);
	LM_STRING_FREE(config.tcpremotehost);
	LM_STRING_FREE(config.tcpremoteinfo);
	LM_STRING_FREE(config.tcprelayclient);
	LM_STRING_FREE(config.helohost);
	LM_STRING_FREE(config.controldir);
	LM_STRING_FREE(config.qmailcontroldir);
	LM_STRING_FREE(config.qmailqueue);
	LM_STRING_FREE(config.logfile);
	LM_STRING_FREE(config.virtualdomains);
	LM_STRING_FREE(config.check_vdomain_prog);
	LM_STRING_FREE(newline);
	LM_STRING_FREE(emptystring);
	LM_STRING_FREE(config.quarantine_ext);

	lmsmtpd_config_free_spam_rules(&global_spam_rules);
	return 0;
}

/* Program globals */
static int lmsmtpd_config_setup_read_string(char *charkey, lm_config_string_t* string, lm_config_string_t defstring)
{
	int retval;
	lm_config_key_t key = NULL;
	lm_config_string_t retstring = NULL;

	LM_STRING_NEW(key, charkey);
	retval = LM_CONFIG_READ_STRING(key, retstring, defstring);
	LM_STRING_FREE(key);

	if(*string != NULL)
		LM_STRING_FREE(*string);
	*string = retstring;
	return retval;
}

static int lmsmtpd_config_setup_read_number(char *charkey, lm_config_number_t* number, lm_config_number_t defnumber)
{
	int retval;
	lm_config_key_t key = NULL;
	lm_config_number_t retnumber;

	LM_STRING_NEW(key, charkey);
	retval = LM_CONFIG_READ_NUMBER(key, retnumber, defnumber);
	LM_STRING_FREE(key);
	*number = retnumber;
	return retval;
}

static int lmsmtpd_config_setup_read_bool(char *charkey, lm_config_bool_t* bool, lm_config_bool_t defbool)
{
	int retval;
	lm_config_key_t key = NULL;
	lm_config_bool_t retbool;

	LM_STRING_NEW(key, charkey);
	retval = LM_CONFIG_READ_BOOL(key, retbool, defbool);
	LM_STRING_FREE(key);
	*bool = retbool;
	return retval;
}

/* Perform checks on variables to make sure the values stored in them
   make sense, else print error message and return error */
int lmsmtpd_config_verify(lmsmtpd_config_t config)
{
	struct stat buf;

	/* Check that the qmailqueue program exists */
	if(lstat(LM_STRING_BUFFER(config.qmailqueue), &buf) != 0)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS, "Executable file specified for qmail-queue does not exist!");
		return -1;
	}

	/* Check that vdomainprog exists and is executable */
	if(lstat(LM_STRING_BUFFER(config.check_vdomain_prog), &buf) != 0)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS, "Executable file specified for virtual domain checking program does not exist!");
		return -1;
	}

	/* check that controldir exists and is readable */
	if(lstat(LM_STRING_BUFFER(config.controldir), &buf) != 0)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS, "Directory file specified for control files does not exist!");

		return -1;
	}
	if(S_ISDIR(buf.st_mode) == 0)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS, "File name specified for control files is not a directory!");
		return -1;
	}

	/* check that qmailcontroldir exists and is readable */
	if(lstat(LM_STRING_BUFFER(config.qmailcontroldir), &buf) != 0)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS, "Directory file specified for qmailcontrol files does not exist!");

		return -1;
	}
	if(S_ISDIR(buf.st_mode) == 0)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS, "File name specified for qmailcontrol files is not a directory!");
		return -1;
	}

	/* Check that max hops is > 0 */
	if(config.maxhops < 0)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS, "maxhops value less than zero!");
		return -1;
	}

	/* Check that max rcpt is > 0 */
	if(config.maxrcpt < 0)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS, "maxrcpt value less than zero!");
		return -1;
	}

	return 0;
}

static int lmsmtpd_config_read(int argc, char **argv)
{
	lm_config_string_t defstring = NULL;
	lm_config_string_t defunknown = NULL;
	lm_config_string_t tcplocaliptmp = NULL;
	lm_config_string_t logdir = NULL;
	lm_config_string_t morercpthosts = NULL;

	/* Set the log file to /dev/null until we know the actual 
	 * log directory, log file name and log level */
	LM_LOG_INIT("/dev/null");

	/* Don't print any log messages until after we've read the debug flag */
	LM_LOG_SET_LEVEL(LM_LOG_LEVEL_NONE);

	/* Open the env config */
	if(LM_CONFIG_ENV_SET_PARAM)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS,
				"could not open environment config -- serious problem!");
	}

	/* Read the log directory if set */
	LM_STRING_NEW(defstring, LM_SMTPD_LOG_DIR_DEFAULT);
	if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_LOG_DIR,
				&logdir, defstring))
		return -1;
	LM_STRING_FREE(defstring);

	/* Read the log file if set */
	LM_STRING_NEW(defstring, LM_SMTPD_LOG_FILE_DEFAULT);
	if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_LOG_FILE,
				&config.logfile, defstring))
		return -1;
	LM_STRING_FREE(defstring);

	LM_STRING_CAT(logdir, config.logfile);
	LM_STRING_FREE(config.logfile);
	config.logfile = logdir;

	/* Read the lmsmptd debug level env variable */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_DEBUG_LEVEL,
				&config.debuglevel, config.debuglevel))
		return -1;

	/* Re init the logger with teh actual log file and set the debug level */
	LM_LOG_INIT(LM_STRING_BUFFER(config.logfile));
	LM_LOG_SET_LEVEL(config.debuglevel);

	/* Read the control directory if set */
	LM_STRING_NEW(defstring, LM_SMTPD_CONTROL_DIR_DEFAULT);
	if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_CONTROL_DIR,
				&config.controldir, defstring))
		return -1;
	LM_STRING_FREE(defstring);

	/* Read the control directory if set */
	LM_STRING_NEW(defstring, LM_SMTPD_QMAIL_QUEUE_DEFAULT);
	if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_QMAIL_QUEUE,
				&config.qmailqueue, defstring))
		return -1;
	LM_STRING_FREE(defstring);

	/* Read the qmail control directory if set */
	if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_QMAIL_CONTROL_DIR,
				&config.qmailcontroldir, config.qmailcontroldir))
		return -1;

	/* Read the max line length */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_MAX_LINE_LENGTH,
				&config.maxlinelen, LM_SMTPD_MAX_LINE_LENGTH_DEFAULT))
		return -1;

	/* Read the max number of hops before flagging error */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_MAX_HOPS,
				&config.maxhops, LM_SMTPD_MAX_HOPS_DEFAULT))
		return -1;

	/* Read the max number of rcpts variable */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_MAX_RCPT,
				&config.maxrcpt, LM_SMTPD_MAX_RCPT_DEFAULT))
		return -1;
	/* RFC Says we have to have at least 100 recipients */
	if(config.maxrcpt < LM_SMTPD_MAX_RCPT_DEFAULT)
	{
		LM_LOG_WARN(LM_SMTPD_LOG_SYS, "setting max rcpt to default...user setting too low for RFC");
		config.maxrcpt = LM_SMTPD_MAX_RCPT_DEFAULT;
	}

	/* Read the number of rcpts to start delaying at */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_RCPT_DELAY_AT,
				&config.rcptdelayat, LM_SMTPD_RCPT_DELAY_AT_DEFAULT))
		return -1;


	/* Read the amount of seconds to delay each rcpt by after DELAY_AT has been met */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_RCPT_DELAY_INC,
				&config.rcptdelayinc, LM_SMTPD_RCPT_DELAY_INC_DEFAULT))
		return -1;

	/* Read the amount amount of seconds to delay each rcpt by after DELAY_AT has been met */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_RCPT_DELAY_MAX,
				&config.rcptdelaymax, LM_SMTPD_RCPT_DELAY_MAX_DEFAULT))
		return -1;


	/* Read whether to enable valid user checking */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_CHECK_VALID_USERS,
				&config.check_valid_users, LM_SMTPD_CHECK_VALID_USERS_DEFAULT))
		return -1;


	/* Read if we should check for superuser owned spam enable files in the user's home directory */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_PER_USER_SPAM_ENABLE,
				&config.per_user_spam_enable, LM_SMTPD_PER_USER_SPAM_ENABLE_DEFAULT))
		return -1;

	/* Read the max virtual domain user verification program variable */
	if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_VDOMAIN_CHECKUSER_PROG,
				&config.check_vdomain_prog, config.check_vdomain_prog))
		return -1;

	/* Read the quarantine email extention if set */
	if(lmsmtpd_config_setup_read_string(LM_SMTPD_ENV_QUARANTINE_EXT,
				&config.quarantine_ext, config.quarantine_ext))
		return -1;

	/* Read the quarantine mail on spam hit flag */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_QUARANTINE_ON_HIT,
				&config.quarantine_on_hit, config.quarantine_on_hit))
		return -1;

	/* Read the quarantine mail on spam hit flag */
	if(lmsmtpd_config_setup_read_number(LM_SMTPD_ENV_RETURN_UDNE_ON_HIT,
				&config.ret_udne_on_hit, config.ret_udne_on_hit))
		return -1;

	/* Close the env config */
	LM_CONFIG_CLOSE;


	/* Open the control directory and read the qmail control files */
	if(LM_CONFIG_CONTROL_SET_PARAM(config.controldir))
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS,
				"could not open config control directory!");
	}

	/* Load the global spam rules from the control file directory */
	if(lmsmtpd_config_load_spam_config(NULL, &global_spam_rules))
		return -1;

	LM_CONFIG_CLOSE;

	/* Open the qmail control directory and read the qmail control files */
	if(LM_CONFIG_CONTROL_SET_PARAM(config.qmailcontroldir))
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS,
				"could not open qmail control directory!");
	}

	/* Setup a blank string for defaults values */
	LM_STRING_NEW(defunknown, "unknown");

	/* Read me from the config */
	if(lmsmtpd_config_setup_read_string(QMAIL_CONTROL_KEY_ME,
				&config.me, emptystring))
		return -1;
	/* This has to be present for qmail to function properly */
	if(LM_STRING_LEN(config.me) == 0)
		return -1;

	/* Read the smtpgreeting from the config */
	if(lmsmtpd_config_setup_read_string(QMAIL_CONTROL_KEY_SMTPGREETING,
				&config.smtpgreeting, config.me))
		return -1;

	/* Read the localiphost from the config */
	if(lmsmtpd_config_setup_read_string(QMAIL_CONTROL_KEY_LOCALIPHOST,
				&config.localiphost, emptystring))
		return -1;

	/* Read the timeoutsmtpd from the config */
	if(lmsmtpd_config_setup_read_number(QMAIL_CONTROL_KEY_TIMEOUTSMTPD,
				&config.timeoutsmtpd, LM_SMTPD_DEFAULT_TIMEOUT))
		return -1;
	/* QMAIL */
	if(config.timeoutsmtpd < 0)
		config.timeoutsmtpd = 1;

	/* Read rcpthosts control file */
	if(lmsmtpd_config_setup_read_string(QMAIL_CONTROL_KEY_RCPTHOSTS,
				&config.rcpthosts, emptystring))
		return -1;

	/* Read MORERCPTHOSTS and cat it onto the end of RCPTHOSTS */
	if(lmsmtpd_config_setup_read_string(QMAIL_CONTROL_KEY_MORERCPTHOSTS,
				&morercpthosts, emptystring))
		return -1;

	LM_STRING_CAT(config.rcpthosts, morercpthosts);
	LM_STRING_FREE(morercpthosts);

	/* Read virtualdomains control file */
	if(lmsmtpd_config_setup_read_string(QMAIL_CONTROL_KEY_VIRTUALDOMAINS,
				&config.virtualdomains, emptystring))
		return -1;

	/* Read badmailfrom control file */
	if(lmsmtpd_config_setup_read_string(QMAIL_CONTROL_KEY_BADMAILFROM,
				&config.badmailfrom, emptystring))
		return -1;

	/* Read the databytes from the config */
	if(lmsmtpd_config_setup_read_number(QMAIL_CONTROL_KEY_DATABYTES,
				&config.databytes, LM_SMTPD_DEFAULT_DATABYTES))
		return -1;

	/* Close the control current config */
	LM_CONFIG_CLOSE;

	/* Open the env config */
	if(LM_CONFIG_ENV_SET_PARAM)
	{
		LM_LOG_ERROR(LM_SMTPD_LOG_SYS,
				"could not open environment config -- serious problem!");
	}

	/* Read the databytes from the environment */
	/* defaults to current value if not present in environment */
	if(lmsmtpd_config_setup_read_number(QMAIL_ENV_KEY_DATABYTES,
				&config.databytes, config.databytes))
		return -1;
	/* QMAIL */
	if(config.databytes == UINT_MAX)
		config.databytes--;

	/* Read the remote ip given by TCP server */
	if(lmsmtpd_config_setup_read_string(QMAIL_ENV_KEY_TCPREMOTEIP,
				&config.tcpremoteip, defunknown))
		return -1;

	/* Set the variable localip to TCPLOCALHOST if set, else set it 
	   to TCPLOCALIP, else set it to unknown */
	/* Read the remote ip given by TCP server */
	if(lmsmtpd_config_setup_read_string(QMAIL_ENV_KEY_TCPLOCALHOST,
				&tcplocaliptmp, defunknown))
		return -1;
	if(lmsmtpd_config_setup_read_string(QMAIL_ENV_KEY_TCPLOCALIP,
				&config.tcplocalip, tcplocaliptmp))
		return -1;
	LM_STRING_FREE(tcplocaliptmp);

	/* Read tcp remote hostname given by TCP server */
	if(lmsmtpd_config_setup_read_string(QMAIL_ENV_KEY_TCPREMOTEHOST,
				&config.tcpremotehost, defunknown))
		return -1;

	/* Read tcp remote host info given by TCP server */
	if(lmsmtpd_config_setup_read_string(QMAIL_ENV_KEY_TCPREMOTEINFO,
				&config.tcpremoteinfo, emptystring))
		return -1;

	/* Read relay client info given by TCP server */
	if(lmsmtpd_config_setup_read_string(QMAIL_ENV_KEY_RELAYCLIENT,
				&config.tcprelayclient, emptystring))
		return -1;

	/* Close the env config */
	LM_CONFIG_CLOSE;

	/* Build the server hello string from the argument given by tcp server */
	LM_STRING_DUP(config.helohost, config.tcpremotehost);
	config.fakehelohost = 0;	/* Real helo host given  */

	/* Free up remaining memory */
	LM_STRING_FREE(defunknown);
	return lmsmtpd_config_verify(config);
}

int lmsmtpd_config_verify_spam_config(spam_rule_config_t *ruledata)
{
	return 0;
}

#define SPAM_CONTROL_SPAM_CHECK_DEFAULT 			0
#define SPAM_CONTROL_SMTP_CHECKS_DEFAULT 			0
#define SPAM_CONTROL_REVERSE_MX_LOOKUP_DEFAULT 		0
#define SPAM_CONTROL_ACCEPT_MAIL_FROM_SELF_DEFAULT 	1
#define SPAM_CONTROL_FROM_VALID_DOMAIN_DEFAULT 		0
#define SPAM_CONTROL_REVERSE_DNS_DEFAULT 			0
#define SPAM_CONTROL_VALID_HELO_DOMAIN_DEFAULT 		0
#define SPAM_CONTROL_BLOCK_ALL_ATTACHMENTS_DEFAULT 	0

/* Special key that can be written by the user to automatically inherit
all global spam rules rather than having to set them all individually.
If this key is not present, the default values are used instead */
#define SPAM_CONTROL_ALL_GLOBAL_RULES "all_global_rules"

#define SPAM_CONTROL_SPAM_CHECK "spam_check"
#define SPAM_CONTROL_SMTP_CHECKS "smtp_checks"
#define SPAM_CONTROL_REVERSE_MX_LOOKUP "reverse_mx_lookup"
#define SPAM_CONTROL_ACCEPT_MAIL_FROM_SELF "accept_mail_from_self"
#define SPAM_CONTROL_FROM_VALID_DOMAIN "from_valid_domain"
#define SPAM_CONTROL_REVERSE_DNS "reverse_dns"
#define SPAM_CONTROL_VALID_HELO_DOMAIN "valid_helo_domain"
#define SPAM_CONTROL_BLOCK_ALL_ATTACHMENTS "block_all_attachments"

#define SPAM_CONTROL_USE_GLOBAL_SERVER_BLACKLIST "use_global_server_blacklist"
#define SPAM_CONTROL_USE_GLOBAL_REGION_BLACKLIST "use_global_region_blacklist"
#define SPAM_CONTROL_USE_GLOBAL_COUNTRY_BLACKLIST "use_global_country_blacklist"
#define SPAM_CONTROL_USE_GLOBAL_CHARSET_BLACKLIST "use_global_charset_blacklist"
#define SPAM_CONTROL_USE_GLOBAL_ATTACHMENT_BLACKLIST "use_global_attachment_blacklist"
#define SPAM_CONTROL_USE_GLOBAL_FROM_BLACKLIST "use_global_from_blacklist"
#define SPAM_CONTROL_USE_GLOBAL_SUBJECT_BLACKLIST "use_global_subject_blacklist"

#define SPAM_CONTROL_USE_GLOBAL_SERVER_WHITELIST "use_global_server_whitelist"
#define SPAM_CONTROL_USE_GLOBAL_REGION_WHITELIST "use_global_region_whitelist"
#define SPAM_CONTROL_USE_GLOBAL_COUNTRY_WHITELIST "use_global_country_whitelist"
#define SPAM_CONTROL_USE_GLOBAL_CHARSET_WHITELIST "use_global_charset_whitelist"
#define SPAM_CONTROL_USE_GLOBAL_ATTACHMENT_WHITELIST "use_global_attachment_whitelist"
#define SPAM_CONTROL_USE_GLOBAL_FROM_WHITELIST "use_global_from_whitelist"
#define SPAM_CONTROL_USE_GLOBAL_SUBJECT_WHITELIST "use_global_subject_whitelist"

#define SPAM_CONTROL_SERVER_WHITELIST "server_whitelist"
#define SPAM_CONTROL_REGION_WHITELIST "region_whitelist"
#define SPAM_CONTROL_COUNTRY_WHITELIST "country_whitelist"
#define SPAM_CONTROL_CHARSET_WHITELIST "charset_whitelist"
#define SPAM_CONTROL_ATTACHMENT_WHITELIST "attachment_whitelist"
#define SPAM_CONTROL_FROM_WHITELIST "from_blacklist"
#define SPAM_CONTROL_SUBJECT_WHITELIST "subject_blacklist"

#define SPAM_CONTROL_SERVER_BLACKLIST "server_blacklist"
#define SPAM_CONTROL_REGION_BLACKLIST "region_blacklist"
#define SPAM_CONTROL_COUNTRY_BLACKLIST "country_blacklist"
#define SPAM_CONTROL_CHARSET_BLACKLIST "charset_blacklist"
#define SPAM_CONTROL_ATTACHMENT_BLACKLIST "attachment_blacklist"
#define SPAM_CONTROL_FROM_BLACKLIST "from_blacklist"
#define SPAM_CONTROL_SUBJECT_BLACKLIST "subject_whitelist"

/* Take a string list and append the global list onto the end of it */
int lmsmtpd_config_add_global_list(lm_config_string_t *list, lm_config_string_t global_list)
{
	int retval = 0;
	lm_config_string_t str = *list;

	/* Add a newline to the end of the whitelist if there
	   isn't one */
	if(LM_STRING_LEN(str) > 0)
		if(LM_STRING_BUFFER(str)[LM_STRING_LEN(str)-1] != '\n')
			retval |= LM_STRING_CAT(str, newline);

	/* cat the global whitelist to the end of the user list */
	if(global_list != NULL)
		retval |= LM_STRING_CAT(str, global_list);
	*list = str;

	return retval;
}

/* Load the configuration files for spam checking, appending the global rule lists 
* if desired */
int lmsmtpd_config_load_spam_config(spam_rule_config_t* globalrules, spam_rule_config_t *ruledata)
{
	int retval = 0;
	lm_config_bool_t false = 0;
	lm_config_bool_t all_global_rules;
	lm_config_bool_t use_global_list;

	LM_ASSERT(ruledata != NULL);

	/* Load the special key to specify to use all global rules */
	if(globalrules != NULL)
	{
		retval |= (lmsmtpd_config_setup_read_bool(SPAM_CONTROL_ALL_GLOBAL_RULES, 
					&all_global_rules, false) != 0);
	}
	else
	{
		/* Passing null for the global rules means to not attempt to 
		 * use them.  This is for using this function to load the global rules
		 * at init time
		 */
		all_global_rules = 0;
	}


	/* If the user specified to use all global rules, set the default values to those
	 * in the globals, else set to the ones specified in the documentation
	 * This only applies to boolean rules here.  The lists are also inherited later in the code.
	 */
	if(all_global_rules == 0)
	{
		ruledata->spam_check = SPAM_CONTROL_SPAM_CHECK_DEFAULT;
		ruledata->smtp_checks = SPAM_CONTROL_SMTP_CHECKS_DEFAULT;
		ruledata->reverse_mx_lookup = SPAM_CONTROL_REVERSE_MX_LOOKUP_DEFAULT;
		ruledata->accept_mail_from_self = SPAM_CONTROL_ACCEPT_MAIL_FROM_SELF_DEFAULT;
		ruledata->from_valid_domain = SPAM_CONTROL_FROM_VALID_DOMAIN_DEFAULT;
		ruledata->reverse_dns = SPAM_CONTROL_REVERSE_DNS_DEFAULT;
		ruledata->valid_helo_domain = SPAM_CONTROL_VALID_HELO_DOMAIN_DEFAULT;
		ruledata->block_all_attachments = SPAM_CONTROL_BLOCK_ALL_ATTACHMENTS_DEFAULT;
	}
	else
	{
		LM_ASSERT(globalrules != NULL);
		ruledata->spam_check = globalrules->spam_check;
		ruledata->smtp_checks = globalrules->smtp_checks;
		ruledata->reverse_mx_lookup = globalrules->reverse_mx_lookup;
		ruledata->accept_mail_from_self = globalrules->accept_mail_from_self;
		ruledata->from_valid_domain = globalrules->from_valid_domain;
		ruledata->reverse_dns = globalrules->reverse_dns;
		ruledata->valid_helo_domain = globalrules->valid_helo_domain;
		ruledata->block_all_attachments = globalrules->block_all_attachments;
	}

	/* Load boolean rules */
	retval |= (lmsmtpd_config_setup_read_bool(SPAM_CONTROL_SPAM_CHECK,
				&(ruledata->spam_check), ruledata->spam_check) != 0 );
	retval |= (lmsmtpd_config_setup_read_bool(SPAM_CONTROL_SMTP_CHECKS, 
				&(ruledata->smtp_checks), ruledata->smtp_checks) != 0 );
	retval |= (lmsmtpd_config_setup_read_bool(SPAM_CONTROL_REVERSE_MX_LOOKUP, 
				&(ruledata->reverse_mx_lookup), ruledata->reverse_mx_lookup) != 0 );
	retval |= (lmsmtpd_config_setup_read_bool(SPAM_CONTROL_ACCEPT_MAIL_FROM_SELF, 
				&(ruledata->accept_mail_from_self), ruledata->accept_mail_from_self) != 0 );
	retval |= (lmsmtpd_config_setup_read_bool(SPAM_CONTROL_FROM_VALID_DOMAIN, 
				&(ruledata->from_valid_domain), ruledata->from_valid_domain) != 0 );
	retval |= (lmsmtpd_config_setup_read_bool(SPAM_CONTROL_REVERSE_DNS, 
				&(ruledata->reverse_dns), ruledata->reverse_dns) != 0 );
	retval |= (lmsmtpd_config_setup_read_bool(SPAM_CONTROL_VALID_HELO_DOMAIN, 
				&(ruledata->valid_helo_domain), ruledata->valid_helo_domain) != 0 );
	retval |= (lmsmtpd_config_setup_read_bool(SPAM_CONTROL_BLOCK_ALL_ATTACHMENTS, 
				&(ruledata->block_all_attachments), ruledata->block_all_attachments) != 0 );

	/* Macro to clean this code up a bit and make it readable */
	/* Load a list, first checking if the use global control file is present 
	 * for the list, then load the list and then append the global list
	 */
#define  LOAD_LIST(_USE_,_NAME_,_LIST_) \
retval |= (lmsmtpd_config_setup_read_string(_NAME_, \
		&(ruledata->_LIST_), emptystring) != 0); \
retval |= (lmsmtpd_config_setup_read_bool(_USE_, \
		&use_global_list, all_global_rules) != 0); \
if((retval == 0) && (use_global_list != 0)) \
	retval |= lmsmtpd_config_add_global_list(&(ruledata->_LIST_), globalrules->_LIST_)

	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_SERVER_WHITELIST, SPAM_CONTROL_SERVER_WHITELIST, server_whitelist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_REGION_WHITELIST, SPAM_CONTROL_REGION_WHITELIST, region_whitelist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_COUNTRY_WHITELIST, SPAM_CONTROL_COUNTRY_WHITELIST, country_whitelist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_CHARSET_WHITELIST, SPAM_CONTROL_CHARSET_WHITELIST, charset_whitelist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_ATTACHMENT_WHITELIST, SPAM_CONTROL_ATTACHMENT_WHITELIST, attachment_whitelist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_FROM_WHITELIST, SPAM_CONTROL_FROM_WHITELIST, from_whitelist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_SUBJECT_WHITELIST, SPAM_CONTROL_SUBJECT_WHITELIST, subject_whitelist);

	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_SERVER_BLACKLIST, SPAM_CONTROL_SERVER_BLACKLIST, server_blacklist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_REGION_BLACKLIST, SPAM_CONTROL_REGION_BLACKLIST, region_blacklist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_COUNTRY_BLACKLIST, SPAM_CONTROL_COUNTRY_BLACKLIST, country_blacklist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_CHARSET_BLACKLIST, SPAM_CONTROL_CHARSET_BLACKLIST, charset_blacklist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_ATTACHMENT_BLACKLIST, SPAM_CONTROL_ATTACHMENT_BLACKLIST, attachment_blacklist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_FROM_BLACKLIST, SPAM_CONTROL_FROM_BLACKLIST, from_blacklist);
	LOAD_LIST(SPAM_CONTROL_USE_GLOBAL_SUBJECT_BLACKLIST, SPAM_CONTROL_SUBJECT_BLACKLIST, subject_blacklist);

	/* Verify that the configuration is ok */
	if(retval == 0)
		retval |= lmsmtpd_config_verify_spam_config(ruledata);

	return retval;
}

int	lmsmtpd_config_load_spam_rules(spam_rule_config_t *ruledata, lm_string_t dotspam)
{
	int retval;

	/* Stat the dotspam file 
	 * The check user functions return a trailing / on filenames if they 
	 * happen to be a directory so we don't have to stat the file again
	 */
	/* If the file is a directory */
	if(LM_STRING_BUFFER(dotspam)[LM_STRING_LEN(dotspam)-1] == '/')
	{
		/* load user rules as control files */
		LM_LOG_MSG(LM_SMTPD_LOG_SYS, "Loading spam rule control files");
		retval = LM_CONFIG_CONTROL_SET_PARAM(dotspam);
	}
	else
	{
		LM_LOG_WARN(LM_SMTPD_LOG_SYS, "This version supports spam control directories only! Spam rules not loaded.");
		return -1;
	}

	/* Load the spam rules for the set config */
	if(retval == 0)
	{
		/* load the user rules and append global rules as per user configuration */
		retval = lmsmtpd_config_load_spam_config(&global_spam_rules, ruledata);

		/* Close the configuration that was opened */
		LM_CONFIG_CLOSE;
	}

	return retval;
}


/* Free all the strings from the ruleset */
int lmsmtpd_config_free_spam_rules(spam_rule_config_t *rules)
{
	if(rules->server_whitelist) LM_STRING_FREE(rules->server_whitelist);
	if(rules->country_whitelist) LM_STRING_FREE(rules->country_whitelist);
	if(rules->region_whitelist) LM_STRING_FREE(rules->region_whitelist);
	if(rules->charset_whitelist) LM_STRING_FREE(rules->charset_whitelist);
	if(rules->attachment_whitelist) LM_STRING_FREE(rules->attachment_whitelist);
	if(rules->from_whitelist) LM_STRING_FREE(rules->from_whitelist);
	if(rules->subject_whitelist) LM_STRING_FREE(rules->subject_whitelist);

	if(rules->server_blacklist) LM_STRING_FREE(rules->server_blacklist);
	if(rules->country_blacklist) LM_STRING_FREE(rules->country_blacklist);
	if(rules->region_blacklist) LM_STRING_FREE(rules->region_blacklist);
	if(rules->charset_blacklist) LM_STRING_FREE(rules->charset_blacklist);
	if(rules->attachment_blacklist) LM_STRING_FREE(rules->attachment_blacklist);
	if(rules->from_blacklist) LM_STRING_FREE(rules->from_blacklist);
	if(rules->subject_blacklist) LM_STRING_FREE(rules->subject_blacklist);

	return 0;
}


